/* 
 *
 * Copyright (c) <2010> Comarch S.A. and/or its subsidiary(-ies).
 * All rights reserved.
 * This component and the accompanying materials are made available
 * under the terms of the License "Eclipse Public License v1.0"
 * which accompanies this distribution, and is available
 * at the URL "http://www.eclipse.org/legal/epl-v10.html".
 *
 * Original Contributors:
 * Comarch S.A. - original contribution.
 *
 * Contributors:
 *
 * Description:
 *
 */

#ifndef HFPCALLTERMINATEPROC_H_
#define HFPCALLTERMINATEPROC_H_

#include <e32base.h>

#include "hfpfeaturecommons.h"

class CHsHFPCommand;
class CHsHFPSettings;

/**
 * @brief Represents "Terminate a call" feature
 */
class CHsHFPCallTerminationProc : public CBase, public MHsHFPProcedureCommons
    {
public:
    /**
     * Two-phased constructor.
     * 
     * @param aObserver feature observer
     * 
     * @return instance of class
     */
    static CHsHFPCallTerminationProc* NewL(
            MHsHFPFeatureProviderObserver* aObserver);

    /**
     * Two-phased constructor.
     * 
     * @param aObserver feature observer
     * 
     * @return instance of class
     */
    static CHsHFPCallTerminationProc* NewLC(
            MHsHFPFeatureProviderObserver* aObserver);

    /** 
     * Destructor
     */
    ~CHsHFPCallTerminationProc();

public:
    //Methods inherited from MHsHFPProcedureCommons
    TInt ProcessCommand(const CHsHFPCommand &aInputCmd,
            CHsHFPCommand &aOutputCmd);

private:
    /**
     * Constructor for performing 1st stage construction
     * 
     * @param aObserver feature observer
     */
    CHsHFPCallTerminationProc(MHsHFPFeatureProviderObserver* aObserver);

    /**
     * Constructor for performing 2nd stage construction
     */
    void ConstructL();

    /**
     * Checks if received CIEV is proper
     * 
     * @param aCommand AT command
     */
    TBool IsProperCIEV(const CHsHFPCommand* aCommand);

private:
    /** Pointer to observer */
    MHsHFPFeatureProviderObserver* iObserver;

    /** Denotes if OK command is expected */
    TBool iWaitingForOK;

    /** Denotes if CIEV command is expected */
    TBool iWaitingForCIEV;

    /** Profile setting */
    CHsHFPSettings* iSettings;
    };

#endif /* HFPCALLTERMINATEPROC_H_ */
